/*   GeBR - An environment for seismic processing.
 *   Copyright (C) 2007-2009 GeBR core team (http://sites.google.com/site/gebrproject/)
 *
 *   This program is free software: you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation, either version 3 of
 *   the License, or * (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program. If not, see
 *   <http://www.gnu.org/licenses/>.
 */

/*
 * File: ui_project_line.c
 * Builds the "Project and Lines" UI and distribute callbacks
 */

#include <string.h>

#include <misc/date.h>
#include <gui/utils.h>

#include "ui_project_line.h"
#include "gebr.h"
#include "support.h"
#include "line.h"
#include "document.h"
#include "project.h"
#include "line.h"
#include "flow.h"
#include "ui_help.h"
#include "callbacks.h"

/*
 * Prototypes
 */

static void
project_line_rename(GtkCellRendererText * cell, gchar * path_string,
		    gchar * new_text, struct ui_project_line * ui_project_line);

static void
project_line_load(void);

static void
project_line_show_help(void);

static void
project_line_on_row_activated(GtkTreeView * tree_view, GtkTreePath * path,
	GtkTreeViewColumn * column, struct ui_project_line * ui_project_line);

static GtkMenu *
project_line_popup_menu(GtkWidget * widget, struct ui_project_line * ui_project_line);


/*
 * Section: Public
 * Public functions.
 */

/*
 * Function: project_line_setup_ui
 * Assembly the project/lines widget.
 *
 * Return:
 * The structure containing relevant data.
 *
 * TODO:
 * Add an info summary about the project/line.
 */
struct ui_project_line *
project_line_setup_ui(void)
{
	struct ui_project_line *	ui_project_line;

	GtkTreeSelection *		selection;
	GtkTreeViewColumn *		col;
	GtkCellRenderer *		renderer;

	GtkWidget *			scrolled_window;
	GtkWidget *			hpanel;
	GtkWidget *			frame;
	GtkWidget *			infopage;

	/* alloc */
	ui_project_line = g_malloc(sizeof(struct ui_project_line));

	/* Create projects/lines ui_project_line->widget */
	ui_project_line->widget = gtk_vbox_new(FALSE, 0);
	hpanel = gtk_hpaned_new();
	gtk_container_add(GTK_CONTAINER(ui_project_line->widget), hpanel);

	/* Left side */
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_paned_pack1(GTK_PANED(hpanel), scrolled_window, FALSE, FALSE);
	gtk_widget_set_size_request(scrolled_window, 300, -1);

	ui_project_line->store = gtk_tree_store_new(PL_N_COLUMN,
		G_TYPE_STRING,  /* Name (title for libgeoxml) */
		G_TYPE_STRING); /* Filename */
	ui_project_line->view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(ui_project_line->store));
	gtk_tree_view_set_popup_callback(GTK_TREE_VIEW(ui_project_line->view),
	        (GtkPopupCallback)project_line_popup_menu, ui_project_line);
	g_signal_connect(ui_project_line->view, "row-activated",
		GTK_SIGNAL_FUNC(project_line_on_row_activated), ui_project_line);
	gtk_container_add(GTK_CONTAINER(scrolled_window), ui_project_line->view);

	/* Projects/lines column */
	renderer = gtk_cell_renderer_text_new();
	g_object_set(renderer, "editable", TRUE, NULL);
	col = gtk_tree_view_column_new_with_attributes(_("Index"), renderer, NULL);
	gtk_tree_view_column_set_sort_column_id(col, PL_TITLE);
	gtk_tree_view_column_set_sort_indicator(col, TRUE);

	gtk_tree_view_append_column(GTK_TREE_VIEW(ui_project_line->view), col);
	gtk_tree_view_column_add_attribute(col, renderer, "text", PL_TITLE);
	g_signal_connect(GTK_OBJECT(renderer), "edited",
			GTK_SIGNAL_FUNC(project_line_rename), ui_project_line);

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ui_project_line->view));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
	g_signal_connect(GTK_OBJECT(ui_project_line->view), "cursor-changed",
			GTK_SIGNAL_FUNC(project_line_load), ui_project_line);

	/* Right side */
	scrolled_window = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_paned_pack2(GTK_PANED(hpanel), scrolled_window, TRUE, FALSE);

	frame = gtk_frame_new(_("Details"));
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), frame);

	infopage = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), infopage);

	/* Title */
	ui_project_line->info.title = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.title), 0, 0);
	gtk_box_pack_start(GTK_BOX(infopage), ui_project_line->info.title, FALSE, TRUE, 0);

	/* Description */
	ui_project_line->info.description = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.description), 0, 0);
	gtk_box_pack_start(GTK_BOX(infopage), ui_project_line->info.description, FALSE, TRUE, 10);

	/* Number of lines */
	ui_project_line->info.numberoflines = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.numberoflines), 0, 0);
	gtk_box_pack_start(GTK_BOX(infopage), ui_project_line->info.numberoflines, FALSE, TRUE, 10);

	GtkWidget *table;
	table = gtk_table_new(4, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(infopage), table, FALSE, TRUE, 0);

	/* Dates */
	ui_project_line->info.created_label = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.created_label), 0, 0);
	gtk_table_attach(GTK_TABLE(table), ui_project_line->info.created_label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 3, 3);

	ui_project_line->info.created = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.created), 0, 0);
	gtk_table_attach(GTK_TABLE(table), ui_project_line->info.created, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 3, 3);

	ui_project_line->info.modified_label = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.modified_label), 0, 0);
	gtk_table_attach(GTK_TABLE(table), ui_project_line->info.modified_label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 3, 3);

	ui_project_line->info.modified = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.modified), 0, 0);
	gtk_table_attach(GTK_TABLE(table), ui_project_line->info.modified, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 3, 3);

	/* Paths for lines */
	ui_project_line->info.path_label = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.path_label), 0, 0);
	gtk_table_attach(GTK_TABLE(table), ui_project_line->info.path_label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 3, 3);

	ui_project_line->info.path1 = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.path1), 0, 0);
	gtk_table_attach(GTK_TABLE(table), ui_project_line->info.path1, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 3, 3);

	ui_project_line->info.path2 = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.path2), 0, 0);
	gtk_table_attach(GTK_TABLE(table), ui_project_line->info.path2, 1, 2, 3, 4, GTK_FILL, GTK_FILL, 3, 3);

	/* Help */
	ui_project_line->info.help = gtk_button_new_from_stock(GTK_STOCK_INFO);
	gtk_box_pack_end(GTK_BOX(infopage), ui_project_line->info.help, FALSE, TRUE, 0);
	g_signal_connect(GTK_OBJECT(ui_project_line->info.help), "clicked",
			 GTK_SIGNAL_FUNC(project_line_show_help), ui_project_line);

	/* Author */
	ui_project_line->info.author = gtk_label_new("");
	gtk_misc_set_alignment(GTK_MISC(ui_project_line->info.author), 0, 0);
	gtk_box_pack_end(GTK_BOX(infopage), ui_project_line->info.author, FALSE, TRUE, 0);

	return ui_project_line;
}

/*
 * Function: project_line_free
 * Frees memory related to project and line
 */
void
project_line_free(void)
{
	if (gebr.project != NULL)
		geoxml_document_free(GEOXML_DOC(gebr.project));
	if (gebr.line != NULL)
		geoxml_document_free(GEOXML_DOC(gebr.line));
	gebr.project_line = NULL;
	gebr.project = NULL;
	gebr.line = NULL;
}


/*
 * Section: Private
 * Private functions.
 */

/*
 * Function: project_line_rename
 * Rename a projet or a line upon double click
 */
static void
project_line_rename(GtkCellRendererText * cell, gchar * path_string, gchar * new_text, struct ui_project_line * ui_project_line)
{
	GtkTreeIter		iter;
	gchar *			old_title;

	gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(ui_project_line->store), &iter, path_string);
	old_title = (gchar *)geoxml_document_get_title(gebr.project_line);

	/* was it really renamed? */
	if (strcmp(old_title, new_text) == 0)
		return;

	/* change it on the xml. */
	geoxml_document_set_title(gebr.project_line, new_text);
	document_save(gebr.project_line);

	/* store's change */
	gtk_tree_store_set(ui_project_line->store, &iter,
			PL_TITLE, new_text,
			-1);

	/* feedback */
	if (geoxml_document_get_type(gebr.project_line) == GEOXML_DOCUMENT_TYPE_PROJECT)
		gebr_message(LOG_INFO, FALSE, TRUE, _("Project '%s' renamed to '%s'"), old_title, new_text);
	else
		gebr_message(LOG_INFO, FALSE, TRUE, _("Line '%s' renamed to '%s'"), old_title, new_text);
	project_line_info_update();
}

/*
 * Function: project_line_load
 * Load a selected project or line from file
 *
 * The selected line or project is loaded from file, upon selection.
 */
static void
project_line_load(void)
{
	GtkTreeSelection *	selection;
	GtkTreeModel *		model;
	GtkTreeIter		iter;
	GtkTreePath *		path;

	gchar *			project_filename;
	gchar *                 project_title;
	gchar *			line_filename;
	gchar *                 line_title;

	gboolean		is_line;

	/* Frees any previous project, line and flow loaded */
	project_line_free();
	flow_free();
	line_load_flows();

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gebr.ui_project_line->view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE) {
		project_line_info_update();
		return;
	}

	path = gtk_tree_model_get_path(GTK_TREE_MODEL(gebr.ui_project_line->store), &iter);
	is_line = gtk_tree_path_get_depth(path) == 2 ? TRUE : FALSE;
	if (is_line == TRUE) {
		GtkTreeIter	child;

		gtk_tree_model_get(GTK_TREE_MODEL(gebr.ui_project_line->store), &iter,
			PL_FILENAME, &line_filename,
			PL_TITLE, &line_title,
			-1);
		child = iter;
		gtk_tree_model_iter_parent(model, &iter, &child);
		gtk_tree_model_get(GTK_TREE_MODEL(gebr.ui_project_line->store), &iter,
			PL_FILENAME, &project_filename,
			PL_TITLE, &project_title,
			-1);

	} else {
		gtk_tree_model_get(GTK_TREE_MODEL(gebr.ui_project_line->store), &iter,
			PL_FILENAME, &project_filename,
			PL_TITLE, &project_title,
			-1);
	}

	gebr.project = GEOXML_PROJECT(document_load(project_filename));
	if (gebr.project == NULL) {
		gebr_message(LOG_ERROR, TRUE, FALSE, _("Unable to load project '%s'"), project_title);
		gebr_message(LOG_ERROR, FALSE, TRUE, _("Unable to load project '%s' from file '%s'"),
			project_title, project_filename);
		goto out;
	}

	if (is_line == TRUE) {
		gebr.line = GEOXML_LINE(document_load(line_filename));
		if (gebr.line == NULL) {
			gebr_message(LOG_ERROR, TRUE, FALSE, _("Unable to load line '%s'"), line_title);
			gebr_message(LOG_ERROR, FALSE, TRUE, _("Unable to load line '%s' from file '%s'"),
				line_title, line_filename);
			project_line_free();
			goto out;
		}

		gebr.project_line = GEOXML_DOC(gebr.line);
		line_load_flows();
	} else {
		gebr.project_line = GEOXML_DOC(gebr.project);
		gebr.line = NULL;
	}

	project_line_info_update();

out:	gtk_tree_path_free(path);
	g_free(project_filename);
	g_free(project_title);
	if (is_line == TRUE) {
		g_free(line_filename);
		g_free(line_title);
	}
}

/*
 * Function: project_line_info_update
 * Update information shown about the selected project or line
 */
void
project_line_info_update(void)
{
	GtkTreeSelection *		selection;
	GtkTreeModel *			model;
	GtkTreeIter			iter;

	gchar *				markup;
	GString *	 		text;

	gboolean			is_project;

	if (gebr.project_line == NULL) {
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.title), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.description), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.created_label), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.created), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.modified_label), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.modified), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path_label), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path1), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path2), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.numberoflines), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.author), "");

		g_object_set(gebr.ui_project_line->info.help, "sensitive", FALSE, NULL);

		navigation_bar_update();
		return;
	}

	/* initialization */
	is_project = (geoxml_document_get_type(gebr.project_line) == GEOXML_DOCUMENT_TYPE_PROJECT) ? TRUE : FALSE;
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gebr.ui_project_line->view));
	gtk_tree_selection_get_selected(selection, &model, &iter);

	/* Title in bold */
	markup = is_project
		? g_markup_printf_escaped("<b>%s %s</b>", _("Project"), geoxml_document_get_title(gebr.project_line))
		: g_markup_printf_escaped("<b>%s %s</b>", _("Line"), geoxml_document_get_title(gebr.project_line));
	gtk_label_set_markup(GTK_LABEL(gebr.ui_project_line->info.title), markup);
	g_free(markup);

	/* Description in italic */
	markup = g_markup_printf_escaped("<i>%s</i>", geoxml_document_get_description(GEOXML_DOC(gebr.project_line)));
	gtk_label_set_markup(GTK_LABEL(gebr.ui_project_line->info.description), markup);
	g_free(markup);

	if (is_project) {
		gint	nlines;

		nlines = gtk_tree_model_iter_n_children(model, &iter);
		markup = nlines != 0
			? nlines == 1
				? g_markup_printf_escaped(_("This project has 1 line"))
				: g_markup_printf_escaped(_("This project has %d lines"), nlines)
			: g_markup_printf_escaped(_("This project has no line"));

		gtk_label_set_markup(GTK_LABEL(gebr.ui_project_line->info.numberoflines), markup);
		g_free(markup);
	} else
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.numberoflines), "");

	/* Date labels */
	markup = g_markup_printf_escaped("<b>%s</b>", _("Created:"));
	gtk_label_set_markup(GTK_LABEL(gebr.ui_project_line->info.created_label), markup);
	g_free(markup);
	markup = g_markup_printf_escaped("<b>%s</b>", _("Modified:"));
	gtk_label_set_markup(GTK_LABEL(gebr.ui_project_line->info.modified_label), markup);
	g_free(markup);

	/* Dates */
	gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.created), localized_date(geoxml_document_get_date_created(gebr.project_line)));
	gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.modified), localized_date(geoxml_document_get_date_modified(gebr.project_line)));

	/* Line's paths */
	if (!is_project) {
		GeoXmlSequence *	path;

		markup = g_markup_printf_escaped("<b>%s</b>", _("Paths:"));
		gtk_label_set_markup(GTK_LABEL(gebr.ui_project_line->info.path_label), markup);
		g_free(markup);

		switch (geoxml_line_get_paths_number(gebr.line)) {
		case 0:
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path1), _("None"));
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path2), "");

			break;
		case 1:
			geoxml_line_get_path(gebr.line, &path, 0);
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path1),
					   geoxml_value_sequence_get(GEOXML_VALUE_SEQUENCE(path)));
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path2), "");

			break;
		case 2:
			geoxml_line_get_path(gebr.line, &path, 0);
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path1),
				geoxml_value_sequence_get(GEOXML_VALUE_SEQUENCE(path)));

			geoxml_line_get_path(gebr.line, &path, 1);
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path2),
				geoxml_value_sequence_get(GEOXML_VALUE_SEQUENCE(path)));

			break;
		default:
			geoxml_line_get_path(gebr.line, &path, 0);
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path1),
					   geoxml_value_sequence_get(GEOXML_VALUE_SEQUENCE(path)));
			gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path2), "...");

			break;
		}
	} else {
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path_label), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path1), "");
		gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.path2), "");
	}

	/* Author and email */
	text = g_string_new(NULL);
	g_string_printf(text, "%s <%s>",
			geoxml_document_get_author(GEOXML_DOC(gebr.project_line)),
			geoxml_document_get_email(GEOXML_DOC(gebr.project_line)));
	gtk_label_set_text(GTK_LABEL(gebr.ui_project_line->info.author), text->str);
	g_string_free(text, TRUE);

	/* Info button */
	g_object_set(gebr.ui_project_line->info.help,
		"sensitive", strlen(geoxml_document_get_help(gebr.project_line)) ? TRUE : FALSE, NULL);

	navigation_bar_update();
}

static void
project_line_show_help(void)
{
	gchar * title;

	title = (geoxml_document_get_type(gebr.project_line) == GEOXML_DOCUMENT_TYPE_PROJECT)
		? _("Project report") : _("Line report");
	help_show(geoxml_document_get_help(gebr.project_line), title);

	return;
}

static void
project_line_on_row_activated(GtkTreeView * tree_view, GtkTreePath * path,
	GtkTreeViewColumn * column, struct ui_project_line * ui_project_line)
{
	gtk_notebook_set_current_page(GTK_NOTEBOOK(gebr.notebook), 1);
}

static GtkMenu *
project_line_popup_menu(GtkWidget * widget, struct ui_project_line * ui_project_line)
{
	GtkTreeSelection *	selection;
	GtkTreeModel *		model;
	GtkTreeIter		iter;

	GtkWidget *		menu;

	menu = gtk_menu_new();

	/* new project */
	gtk_container_add(GTK_CONTAINER(menu), gtk_action_create_menu_item(
		gtk_action_group_get_action(gebr.action_group, "project_line_new_project")));

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(gebr.ui_project_line->view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
		goto out;

	/* new line */
	gtk_container_add(GTK_CONTAINER(menu), gtk_action_create_menu_item(
		gtk_action_group_get_action(gebr.action_group, "project_line_new_line")));
	/* properties */
	gtk_container_add(GTK_CONTAINER(menu), gtk_action_create_menu_item(
		gtk_action_group_get_action(gebr.action_group, "project_line_properties")));
	/* paths */
	if (gebr.line != NULL)
		gtk_container_add(GTK_CONTAINER(menu), gtk_action_create_menu_item(
			gtk_action_group_get_action(gebr.action_group, "project_line_line_paths")));
	/* delete */
	gtk_container_add(GTK_CONTAINER(menu),
		gtk_action_create_menu_item(gtk_action_group_get_action(gebr.action_group, "project_line_delete")));

out:	gtk_widget_show_all(menu);

	return GTK_MENU(menu);
}
